@@ -13,7 +13,6 @@ MACDEPLOYQT=$1
 QTDIR="`dirname $MACDEPLOYQT`/.."
 LIBRARY_NAME="libQupZilla.2.dylib"
 PLUGINS="QupZilla.app/Contents/Resources/plugins"
-QTPLUGINS="QupZilla.app/Contents/PlugIns"
 
 # cd to directory with bundle
 test -d bin || cd ..
@@ -35,21 +34,6 @@ do
  install_name_tool -change $LIBRARY_NAME @executable_path/$LIBRARY_NAME $plugin
 done
 
-# copy known, missing, Qt native library plugins into bundle
-#
-# See:
-#  *  http://code.qt.io/cgit/qt/qttools.git/tree/src/macdeployqt/shared/shared.cpp#n1044
-#
-mkdir -p $QTPLUGINS
-
-FILE="$QTDIR/plugins/iconengines/libqsvgicon.dylib"
-if [ -f "$FILE" ]; then
- cp $FILE $QTPLUGINS/
-else
- echo "$FILE: No such file"
- exit 1
-fi
-
 # run macdeployqt
 $MACDEPLOYQT QupZilla.app -qmldir=$PWD/../src/lib/data/data
 
