@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include <QPointer>
 #include <QUrl>
 
-#include "restoremanager.h"
+#include "webtab.h"
 #include "qzcommon.h"
 
 class QLabel;
@@ -41,9 +41,8 @@ class BookmarksMenu;
 class BookmarksToolbar;
 class AutoFill;
 class MainApplication;
-class WebTab;
 class WebView;
-class AdBlockIcon;
+class WebPage;
 class SideBar;
 class SideBarManager;
 class ProgressBar;
@@ -58,14 +57,31 @@ class QUPZILLA_EXPORT BrowserWindow : public QMainWindow
     Q_OBJECT
 
 public:
+    struct SavedWindow {
+        QByteArray windowState;
+        QByteArray windowGeometry;
+        QHash<QString, QVariant> windowUiState;
+        int virtualDesktop = -1;
+        int currentTab = -1;
+        QVector<WebTab::SavedTab> tabs;
+
+        SavedWindow();
+        SavedWindow(BrowserWindow *window);
+
+        bool isValid() const;
+        void clear();
+
+        friend QUPZILLA_EXPORT QDataStream &operator<<(QDataStream &stream, const SavedWindow &window);
+        friend QUPZILLA_EXPORT QDataStream &operator>>(QDataStream &stream, SavedWindow &window);
+    };
+
     explicit BrowserWindow(Qz::BrowserWindowType type, const QUrl &url = QUrl());
     ~BrowserWindow();
 
     void setStartTab(WebTab* tab);
     void setStartPage(WebPage* page);
 
-    void restoreWindowState(const RestoreManager::WindowData &d);
-    void saveSideBarWidth();
+    void restoreWindow(const SavedWindow &window);
 
     bool fullScreenNavigationVisible() const;
     void showNavigationWithFullScreen();
@@ -84,10 +100,9 @@ public:
     void removeActions(const QList<QAction*> &actions);
 
     SideBar* addSideBar();
+    void saveSideBarSettings();
 
-    QByteArray saveState(int version = 0) const;
-    bool restoreState(const QByteArray &state, int version = 0);
-
+    int tabCount() const;
     TabbedWebView* weView() const;
     TabbedWebView* weView(int index) const;
 
@@ -99,7 +114,6 @@ public:
     NavigationBar* navigationBar() const;
     SideBarManager* sideBarManager() const;
     QLabel* ipLabel() const;
-    AdBlockIcon* adBlockIcon() const;
     QMenu* superMenu() const;
 
     QUrl homepageUrl() const;
@@ -141,8 +155,6 @@ public slots:
     void loadAddress(const QUrl &url);
     void showSource(WebView *view = Q_NULLPTR);
 
-    void showNormal();
-
 private slots:
     void addTab();
     void openLocation();
@@ -172,10 +184,12 @@ private:
     void setupMenu();
     void updateStartupFocus();
 
-    QAction *createEncodingAction(const QString &codecName, const QString &activeCodecName,
-                                  QMenu *menu);
+    QAction *createEncodingAction(const QString &codecName, const QString &activeCodecName, QMenu *menu);
     void createEncodingSubMenu(const QString &name, QStringList &codecNames, QMenu *menu);
 
+    QHash<QString, QVariant> saveUiState();
+    void restoreUiState(const QHash<QString, QVariant> &state);
+
     QUrl m_startUrl;
     QUrl m_homepage;
     Qz::BrowserWindowType m_windowType;
@@ -185,8 +199,6 @@ private:
     QVBoxLayout* m_mainLayout;
     QSplitter* m_mainSplitter;
 
-    AdBlockIcon* m_adblockIcon;
-
     TabWidget* m_tabWidget;
     QPointer<SideBar> m_sideBar;
     SideBarManager* m_sideBarManager;
@@ -214,8 +226,6 @@ private:
     bool m_menuBarVisible;
     bool m_statusBarVisible;
     bool m_isHtmlFullScreen;
-    Qt::WindowStates m_oldWindowState = Qt::WindowNoState;
-    Qt::WindowStates m_normalWindowState = Qt::WindowNoState;
     QTimer* m_hideNavigationTimer;
 
     QList<QPointer<QWidget> > m_deleteOnCloseWidgets;
