@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -80,7 +80,7 @@ void LocationCompleter::complete(const QString &string)
     connect(job, SIGNAL(finished()), this, SLOT(refreshJobFinished()));
     connect(this, SIGNAL(cancelRefreshJob()), job, SLOT(jobCancelled()));
 
-    if (qzSettings->searchFromAddressBar && trimmedStr.length() > 2) {
+    if (qzSettings->searchFromAddressBar && qzSettings->showABSearchSuggestions && trimmedStr.length() >= 2) {
         if (!m_openSearchEngine) {
             m_openSearchEngine = new OpenSearchEngine(this);
             m_openSearchEngine->setNetworkAccessManager(mApp->networkManager());
@@ -90,6 +90,8 @@ void LocationCompleter::complete(const QString &string)
         m_openSearchEngine->setSuggestionsParameters(LocationBar::searchEngine().suggestionsParameters);
         m_suggestionsTerm = trimmedStr;
         m_openSearchEngine->requestSuggestions(m_suggestionsTerm);
+    } else {
+        m_oldSuggestions.clear();
     }
 }
 
@@ -121,10 +123,10 @@ void LocationCompleter::refreshJobFinished()
 
         if (qzSettings->useInlineCompletion) {
             emit showDomainCompletion(job->domainCompletion());
-
-            m_originalText = m_locationBar->text();
-            s_view->setOriginalText(m_originalText);
         }
+
+        m_originalText = m_locationBar->text();
+        s_view->setOriginalText(m_originalText);
     }
 
     job->deleteLater();
@@ -282,10 +284,11 @@ void LocationCompleter::indexDeleteRequested(const QModelIndex &index)
     if (index.data(LocationCompleterModel::BookmarkRole).toBool()) {
         BookmarkItem* bookmark = static_cast<BookmarkItem*>(index.data(LocationCompleterModel::BookmarkItemRole).value<void*>());
         mApp->bookmarks()->removeBookmark(bookmark);
-    }
-    else {
+    } else if (index.data(LocationCompleterModel::HistoryRole).toBool()) {
         int id = index.data(LocationCompleterModel::IdRole).toInt();
         mApp->history()->deleteHistoryEntry(id);
+    } else {
+        return;
     }
 
     s_view->setUpdatesEnabled(false);
@@ -337,6 +340,7 @@ void LocationCompleter::loadString(const QString &urlString)
 
 void LocationCompleter::showPopup()
 {
+    Q_ASSERT(m_window);
     Q_ASSERT(m_locationBar);
 
     if (!m_locationBar->hasFocus() || s_model->rowCount() == 0) {
@@ -363,10 +367,8 @@ void LocationCompleter::showPopup()
     connect(s_view, SIGNAL(indexDeleteRequested(QModelIndex)), this, SLOT(indexDeleteRequested(QModelIndex)));
     connect(s_view->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), this, SLOT(currentChanged(QModelIndex)));
 
-    if (m_locationBar->nativeParentWidget()) {
-        s_view->createWinId();
-        s_view->windowHandle()->setTransientParent(m_locationBar->nativeParentWidget()->windowHandle());
-    }
+    s_view->createWinId();
+    s_view->windowHandle()->setTransientParent(m_window->windowHandle());
 
     adjustPopupSize();
 }
@@ -376,8 +378,10 @@ void LocationCompleter::adjustPopupSize()
     const int maxItemsCount = 12;
     const int popupHeight = s_view->sizeHintForRow(0) * qMin(maxItemsCount, s_model->rowCount()) + 2 * s_view->frameWidth();
 
-    m_originalText = m_locationBar->text();
-    s_view->setOriginalText(m_originalText);
+    if (s_view->currentIndex().row() == 0) {
+        m_originalText = m_locationBar->text();
+        s_view->setOriginalText(m_originalText);
+    }
 
     s_view->resize(s_view->width(), popupHeight);
     s_view->show();
