@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -117,6 +117,9 @@ void LocationCompleterDelegate::paint(QPainter* painter, const QStyleOptionViewI
     }
 
     QString searchText = index.data(LocationCompleterModel::SearchStringRole).toString();
+    if (index.data(LocationCompleterModel::VisitSearchItemRole).toBool()) {
+        searchText.clear();
+    }
 
     // Draw title
     leftPosition += 2;
