@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@
 #include "networkmanager.h"
 #include "desktopnotificationsfactory.h"
 #include "desktopnotification.h"
-#include "navigationbar.h"
 #include "thememanager.h"
 #include "acceptlanguage.h"
 #include "qztools.h"
@@ -216,11 +215,6 @@ Preferences::Preferences(BrowserWindow* window)
     connect(ui->instantBookmarksToolbar, SIGNAL(toggled(bool)), ui->showBookmarksToolbar, SLOT(setDisabled(bool)));
     connect(ui->showBookmarksToolbar, SIGNAL(toggled(bool)), ui->instantBookmarksToolbar, SLOT(setDisabled(bool)));
     ui->showNavigationToolbar->setChecked(settings.value("showNavigationToolbar", true).toBool());
-    ui->showHome->setChecked(settings.value("showHomeButton", true).toBool());
-    ui->showBackForward->setChecked(settings.value("showBackForwardButtons", true).toBool());
-    ui->showAddTabButton->setChecked(settings.value("showAddTabButton", false).toBool());
-    ui->showReloadStopButtons->setChecked(settings.value("showReloadButton", true).toBool());
-    ui->showWebSearchBar->setChecked(settings.value("showWebSearchBar", true).toBool());
     int currentSettingsPage = settings.value("settingsDialogPage", 0).toInt(0);
     settings.endGroup();
 
@@ -265,6 +259,8 @@ Preferences::Preferences(BrowserWindow* window)
     ui->searchFromAddressBar->setChecked(searchFromAB);
     ui->searchWithDefaultEngine->setEnabled(searchFromAB);
     ui->searchWithDefaultEngine->setChecked(settings.value("SearchWithDefaultEngine", false).toBool());
+    ui->showABSearchSuggestions->setEnabled(searchFromAB);
+    ui->showABSearchSuggestions->setChecked(settings.value("showSearchSuggestions", true).toBool());
     connect(ui->searchFromAddressBar, SIGNAL(toggled(bool)), this, SLOT(searchFromAddressBarChanged(bool)));
     settings.endGroup();
 
@@ -348,7 +344,7 @@ Preferences::Preferences(BrowserWindow* window)
 
     //FONTS
     settings.beginGroup("Browser-Fonts");
-    QWebEngineSettings* webSettings = QWebEngineSettings::defaultSettings();
+    QWebEngineSettings* webSettings = mApp->webSettings();
     auto defaultFont = [&](QWebEngineSettings::FontFamily font) -> const QString {
         const QString family = webSettings->fontFamily(font);
         if (!family.isEmpty())
@@ -701,6 +697,7 @@ void Preferences::setManualProxyConfigurationEnabled(bool state)
 void Preferences::searchFromAddressBarChanged(bool stat)
 {
     ui->searchWithDefaultEngine->setEnabled(stat);
+    ui->showABSearchSuggestions->setEnabled(stat);
 }
 
 void Preferences::saveHistoryChanged(bool stat)
@@ -757,7 +754,7 @@ void Preferences::newTabChanged(int value)
 
 void Preferences::afterLaunchChanged(int value)
 {
-    ui->dontLoadTabsUntilSelected->setEnabled(value == 3);
+    ui->dontLoadTabsUntilSelected->setEnabled(value == 3 || value == 4);
 }
 
 void Preferences::changeCachePathClicked()
@@ -903,11 +900,6 @@ void Preferences::saveSettings()
     settings.setValue("instantBookmarksToolbar", ui->instantBookmarksToolbar->isChecked());
     settings.setValue("showBookmarksToolbar", ui->showBookmarksToolbar->isChecked());
     settings.setValue("showNavigationToolbar", ui->showNavigationToolbar->isChecked());
-    settings.setValue("showHomeButton", ui->showHome->isChecked());
-    settings.setValue("showBackForwardButtons", ui->showBackForward->isChecked());
-    settings.setValue("showWebSearchBar", ui->showWebSearchBar->isChecked());
-    settings.setValue("showAddTabButton", ui->showAddTabButton->isChecked());
-    settings.setValue("showReloadButton", ui->showReloadStopButtons->isChecked());
     settings.endGroup();
 
     //TABS
@@ -1039,6 +1031,7 @@ void Preferences::saveSettings()
     settings.beginGroup("SearchEngines");
     settings.setValue("SearchFromAddressBar", ui->searchFromAddressBar->isChecked());
     settings.setValue("SearchWithDefaultEngine", ui->searchWithDefaultEngine->isChecked());
+    settings.setValue("showSearchSuggestions", ui->showABSearchSuggestions->isChecked());
     settings.endGroup();
 
     //Languages
@@ -1073,7 +1066,6 @@ void Preferences::saveSettings()
     mApp->cookieJar()->loadSettings();
     mApp->history()->loadSettings();
     mApp->reloadSettings();
-    mApp->plugins()->c2f_saveSettings();
     mApp->desktopNotifications()->loadSettings();
     mApp->autoFill()->loadSettings();
     mApp->networkManager()->loadSettings();
