@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -65,32 +65,6 @@ void AddTabButton::mouseReleaseEvent(QMouseEvent* event)
     ToolButton::mouseReleaseEvent(event);
 }
 
-void AddTabButton::dragEnterEvent(QDragEnterEvent* event)
-{
-    const QMimeData* mime = event->mimeData();
-
-    if (mime->hasUrls()) {
-        event->acceptProposedAction();
-        return;
-    }
-
-    ToolButton::dragEnterEvent(event);
-}
-
-void AddTabButton::dropEvent(QDropEvent* event)
-{
-    const QMimeData* mime = event->mimeData();
-
-    if (!mime->hasUrls()) {
-        ToolButton::dropEvent(event);
-        return;
-    }
-
-    foreach (const QUrl &url, mime->urls()) {
-        m_tabWidget->addView(url, Qz::NT_SelectedNewEmptyTab);
-    }
-}
-
 void MenuTabs::mouseReleaseEvent(QMouseEvent* event)
 {
     if (event->button() == Qt::MiddleButton) {
@@ -138,6 +112,7 @@ TabWidget::TabWidget(BrowserWindow* window, QWidget* parent)
 
     // AddTab button displayed next to last tab
     m_buttonAddTab = new AddTabButton(this, m_tabBar);
+    m_buttonAddTab->setProperty("outside-tabbar", false);
     connect(m_buttonAddTab, SIGNAL(clicked()), m_window, SLOT(addTab()));
 
     // AddTab button displayed outside tabbar (as corner widget)
@@ -218,11 +193,7 @@ bool TabWidget::validIndex(int index) const
 
 void TabWidget::updateClosedTabsButton()
 {
-    if (!m_showClosedTabsButton) {
-        m_buttonClosedTabs->hide();
-    }
-
-    m_buttonClosedTabs->setEnabled(canRestoreTab());
+    m_buttonClosedTabs->setVisible(m_showClosedTabsButton && canRestoreTab());
 }
 
 bool TabWidget::isCurrentTabFresh() const
@@ -243,7 +214,6 @@ void TabWidget::tabBarOverFlowChanged(bool overflowed)
     // Show buttons displayed outside tabbar (corner widgets)
     m_buttonAddTab2->setVisible(overflowed);
     m_buttonListTabs->setVisible(overflowed);
-    m_buttonClosedTabs->setVisible(m_showClosedTabsButton);
 }
 
 void TabWidget::moveAddTabButton(int posX)
@@ -293,13 +263,11 @@ void TabWidget::aboutToShowClosedTabsMenu()
 {
     m_menuClosedTabs->clear();
 
-    int i = 0;
-    const QLinkedList<ClosedTabsManager::Tab> closedTabs = closedTabsManager()->allClosedTabs();
-
-    foreach (const ClosedTabsManager::Tab &tab, closedTabs) {
-        const QString title = QzTools::truncatedText(tab.title, 40);
-        QAction* act = m_menuClosedTabs->addAction(tab.icon, title, this, SLOT(restoreClosedTab()));
-        act->setData(i++);
+    const auto closedTabs = closedTabsManager()->closedTabs();
+    for (int i = 0; i < closedTabs.count(); ++i) {
+        const ClosedTabsManager::Tab tab = closedTabs.at(i);
+        const QString title = QzTools::truncatedText(tab.tabState.title, 40);
+        m_menuClosedTabs->addAction(tab.tabState.icon, title, this, SLOT(restoreClosedTab()))->setData(i);
     }
 
     if (m_menuClosedTabs->isEmpty()) {
@@ -356,6 +324,7 @@ int TabWidget::addView(const LoadRequest &req, const QString &title, const Qz::N
     }
 
     WebTab* webTab = new WebTab(m_window);
+    webTab->setPinned(pinned);
     webTab->locationBar()->showUrl(url);
     m_locationBars->addWidget(webTab->locationBar());
 
@@ -403,17 +372,34 @@ int TabWidget::addView(const LoadRequest &req, const QString &title, const Qz::N
     return index;
 }
 
-int TabWidget::addView(WebTab* tab)
+int TabWidget::addView(WebTab *tab, const Qz::NewTabPositionFlags &openFlags)
+{
+    return insertView(count() + 1, tab, openFlags);
+}
+
+int TabWidget::insertView(int index, WebTab *tab, const Qz::NewTabPositionFlags &openFlags)
 {
     m_locationBars->addWidget(tab->locationBar());
-    int index = addTab(tab, QString());
+    int newIndex = insertTab(index, tab, QString(), tab->isPinned());
     tab->attach(m_window);
 
+    if (openFlags.testFlag(Qz::NT_SelectedTab)) {
+        setCurrentIndex(newIndex);
+    } else {
+        m_lastBackgroundTabIndex = index;
+    }
+
     connect(tab->webView(), SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
     connect(tab->webView(), SIGNAL(urlChanged(QUrl)), this, SIGNAL(changed()));
     connect(tab->webView(), SIGNAL(ipChanged(QString)), m_window->ipLabel(), SLOT(setText(QString)));
 
-    return index;
+    // Make sure user notice opening new background tabs
+    if (!(openFlags & Qz::NT_SelectedTab)) {
+        m_tabBar->ensureVisible(index);
+    }
+
+    emit changed();
+    return newIndex;
 }
 
 void TabWidget::addTabFromClipboard()
@@ -435,12 +421,15 @@ void TabWidget::closeTab(int index)
     if (!webTab || !validIndex(index))
         return;
 
+    // This is already handled in requestCloseTab
+    if (count() <= 1) {
+        requestCloseTab(index);
+        return;
+    }
+
+    m_closedTabsManager->saveTab(webTab);
+
     TabbedWebView *webView = webTab->webView();
-
-    // Save tab url and history
-    if (webView->url().toString() != QL1S("qupzilla:restore"))
-        m_closedTabsManager->saveTab(webTab, index);
-
     m_locationBars->removeWidget(webView->webTab()->locationBar());
     disconnect(webView, SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
     disconnect(webView, SIGNAL(urlChanged(QUrl)), this, SIGNAL(changed()));
@@ -448,11 +437,6 @@ void TabWidget::closeTab(int index)
 
     m_lastBackgroundTabIndex = -1;
 
-    if (m_menuTabs->isVisible()) {
-        QAction* labelAction = m_menuTabs->actions().last();
-        labelAction->setText(tr("Currently you have %n opened tab(s)", "", count() - 1));
-    }
-
     removeTab(index);
     webTab->deleteLater();
 
@@ -472,12 +456,8 @@ void TabWidget::requestCloseTab(int index)
 
     TabbedWebView *webView = webTab->webView();
 
-    // Don't close restore page!
-    if (webView->url().toString() == QL1S("qupzilla:restore") && mApp->restoreManager())
-        return;
-
     // This would close last tab, so we close the window instead
-    if (count() == 1) {
+    if (count() <= 1) {
         // If we are not closing window upon closing last tab, let's just load new-tab-url
         if (m_dontCloseWithOneTab) {
             if (webView->url() == m_urlOnNewTab) {
@@ -485,6 +465,7 @@ void TabWidget::requestCloseTab(int index)
                 // to close the last tab multiple times
                 m_closedTabsManager->takeLastClosedTab();
             }
+            webView->zoomReset();
             webView->load(m_urlOnNewTab);
             return;
         }
@@ -661,17 +642,17 @@ void TabWidget::detachTab(WebTab* tab)
     disconnect(tab->webView(), SIGNAL(ipChanged(QString)), m_window->ipLabel(), SLOT(setText(QString)));
 
     tab->detach();
+
+    if (count() == 0) {
+        m_window->close();
+    }
 }
 
 void TabWidget::detachTab(int index)
 {
     WebTab* tab = weTab(index);
-
-    if (tab->isPinned() || count() == 1) {
-        return;
-    }
-
     detachTab(tab);
+    tab->setPinned(false);
 
     BrowserWindow* window = mApp->createWindow(Qz::BW_NewWindow);
     window->setStartTab(tab);
@@ -685,7 +666,7 @@ int TabWidget::duplicateTab(int index)
 
     WebTab* webTab = weTab(index);
 
-    int id = addView(QUrl(), webTab->title(), Qz::NT_CleanNotSelectedTab);
+    int id = addView(QUrl(), webTab->title(), Qz::NT_CleanSelectedTab);
     weTab(id)->p_restoreTab(webTab->url(), webTab->historyData(), webTab->zoomLevel());
 
     return id;
@@ -715,9 +696,9 @@ void TabWidget::restoreClosedTab(QObject* obj)
         return;
     }
 
-    int index = addView(QUrl(), tab.title, Qz::NT_CleanSelectedTab, false, tab.position);
+    int index = addView(QUrl(), tab.tabState.title, Qz::NT_CleanSelectedTab, false, tab.position);
     WebTab* webTab = weTab(index);
-    webTab->p_restoreTab(tab.url, tab.history, tab.zoomLevel);
+    webTab->p_restoreTab(tab.tabState);
 
     updateClosedTabsButton();
 }
@@ -728,12 +709,11 @@ void TabWidget::restoreAllClosedTabs()
         return;
     }
 
-    const QLinkedList<ClosedTabsManager::Tab> &closedTabs = m_closedTabsManager->allClosedTabs();
-
-    foreach (const ClosedTabsManager::Tab &tab, closedTabs) {
-        int index = addView(QUrl(), tab.title, Qz::NT_CleanSelectedTab);
+    const auto closedTabs = m_closedTabsManager->closedTabs();
+    for (const ClosedTabsManager::Tab &tab : closedTabs) {
+        int index = addView(QUrl(), tab.tabState.title, Qz::NT_CleanSelectedTab);
         WebTab* webTab = weTab(index);
-        webTab->p_restoreTab(tab.url, tab.history, tab.zoomLevel);
+        webTab->p_restoreTab(tab.tabState);
     }
 
     clearClosedTabsList();
@@ -741,7 +721,7 @@ void TabWidget::restoreAllClosedTabs()
 
 void TabWidget::clearClosedTabsList()
 {
-    m_closedTabsManager->clearList();
+    m_closedTabsManager->clearClosedTabs();
     updateClosedTabsButton();
 }
 
@@ -780,50 +760,16 @@ QList<WebTab*> TabWidget::allTabs(bool withPinned)
     return allTabs;
 }
 
-QByteArray TabWidget::saveState()
-{
-    int currentTabIndex = 0;
-    QVector<WebTab::SavedTab> tabList;
-
-    for (int i = 0; i < count(); ++i) {
-        WebTab* webTab = weTab(i);
-        if (!webTab)
-            continue;
-
-        WebTab::SavedTab tab(webTab);
-        if (!tab.isValid())
-            continue;
-
-        tabList.append(tab);
-
-        if (webTab->isCurrentTab())
-            currentTabIndex = tabList.size() - 1;
-    }
-
-    QByteArray data;
-    QDataStream stream(&data, QIODevice::WriteOnly);
-
-    stream << tabList.count();
-
-    foreach (const WebTab::SavedTab &tab, tabList) {
-        stream << tab;
-    }
-
-    stream << currentTabIndex;
-
-    return data;
-}
-
 bool TabWidget::restoreState(const QVector<WebTab::SavedTab> &tabs, int currentTab)
 {
+    if (tabs.isEmpty()) {
+        return false;
+    }
+
     for (int i = 0; i < tabs.size(); ++i) {
         WebTab::SavedTab tab = tabs.at(i);
-
         int index = addView(QUrl(), Qz::NT_CleanSelectedTab, false, tab.isPinned);
         weTab(index)->restoreTab(tab);
-
-        if (tab.isPinned)
-            m_tabBar->updatePinnedTabCloseButton(index);
     }
 
     setCurrentIndex(currentTab);
@@ -834,15 +780,6 @@ bool TabWidget::restoreState(const QVector<WebTab::SavedTab> &tabs, int currentT
     return true;
 }
 
-void TabWidget::closeRecoveryTab()
-{
-    foreach (WebTab* tab, allTabs(false)) {
-        if (tab->url().toString() == QLatin1String("qupzilla:restore")) {
-            closeTab(tab->tabIndex());
-        }
-    }
-}
-
 TabWidget::~TabWidget()
 {
     delete m_closedTabsManager;
