@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2017  David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -44,8 +44,6 @@ public:
 private:
     void wheelEvent(QWheelEvent* event);
     void mouseReleaseEvent(QMouseEvent* event);
-    void dragEnterEvent(QDragEnterEvent* event);
-    void dropEvent(QDropEvent* event);
 
     TabBar* m_tabBar;
     TabWidget* m_tabWidget;
@@ -71,9 +69,7 @@ public:
     explicit TabWidget(BrowserWindow* mainclass, QWidget* parent = 0);
     ~TabWidget();
 
-    QByteArray saveState();
     bool restoreState(const QVector<WebTab::SavedTab> &tabs, int currentTab);
-    void closeRecoveryTab();
 
     void setCurrentIndex(int index);
 
@@ -102,7 +98,8 @@ public:
 public slots:
     int addView(const LoadRequest &req, const Qz::NewTabPositionFlags &openFlags, bool selectLine = false, bool pinned = false);
     int addView(const LoadRequest &req, const QString &title = tr("New tab"), const Qz::NewTabPositionFlags &openFlags = Qz::NT_SelectedTab, bool selectLine = false, int position = -1, bool pinned = false);
-    int addView(WebTab* tab);
+    int addView(WebTab *tab, const Qz::NewTabPositionFlags &openFlags);
+    int insertView(int index, WebTab *tab, const Qz::NewTabPositionFlags &openFlags);
 
     void addTabFromClipboard();
     int duplicateTab(int index);
