@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -114,26 +114,25 @@ QIcon WebView::icon(bool allowNull) const
     return IconProvider::iconForUrl(url(), allowNull);
 }
 
-QString WebView::title() const
+QString WebView::title(bool allowEmpty) const
 {
     QString title = QWebEngineView::title();
 
+    if (allowEmpty) {
+        return title;
+    }
+
     if (title.isEmpty()) {
         title = url().toString(QUrl::RemoveFragment);
     }
 
-    if (title.isEmpty() || title == QLatin1String("about:blank")) {
+    if (title.isEmpty() || title == QL1S("about:blank")) {
         return tr("Empty Page");
     }
 
     return title;
 }
 
-bool WebView::isTitleEmpty() const
-{
-    return QWebEngineView::title().isEmpty();
-}
-
 WebPage* WebView::page() const
 {
     return m_page;
@@ -154,7 +153,8 @@ void WebView::setPage(WebPage *page)
     m_page->setParent(this);
     QWebEngineView::setPage(m_page);
 
-    connect(m_page, SIGNAL(privacyChanged(bool)), this, SIGNAL(privacyChanged(bool)));
+    connect(m_page, &WebPage::privacyChanged, this, &WebView::privacyChanged);
+    connect(m_page, &WebPage::printRequested, this, &WebView::printPage);
 
     // Set default zoom level
     zoomReset();
@@ -165,11 +165,16 @@ void WebView::setPage(WebPage *page)
     // Scrollbars must be added only after QWebEnginePage is set
     WebScrollBarManager::instance()->addWebView(this);
 
+    emit pageChanged(m_page);
     mApp->plugins()->emitWebPageCreated(m_page);
 }
 
 void WebView::load(const QUrl &url)
 {
+    if (m_page && !m_page->acceptNavigationRequest(url, QWebEnginePage::NavigationTypeTyped, true)) {
+        return;
+    }
+
     QWebEngineView::load(url);
 
     if (!m_firstLoad) {
@@ -558,7 +563,7 @@ void WebView::showSiteInfo()
 
 void WebView::searchSelectedText()
 {
-    SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
+    SearchEngine engine = mApp->searchEnginesManager()->defaultEngine();
     if (QAction* act = qobject_cast<QAction*>(sender())) {
         if (act->data().isValid()) {
             engine = act->data().value<SearchEngine>();
@@ -571,7 +576,7 @@ void WebView::searchSelectedText()
 
 void WebView::searchSelectedTextInBackgroundTab()
 {
-    SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
+    SearchEngine engine = mApp->searchEnginesManager()->defaultEngine();
     if (QAction* act = qobject_cast<QAction*>(sender())) {
         if (act->data().isValid()) {
             engine = act->data().value<SearchEngine>();
@@ -888,7 +893,7 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const WebHitTestResult
     // KDE is displaying newlines in menu actions ... weird -,-
     selectedText.replace(QLatin1Char('\n'), QLatin1Char(' ')).replace(QLatin1Char('\t'), QLatin1Char(' '));
 
-    SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
+    SearchEngine engine = mApp->searchEnginesManager()->defaultEngine();
     Action* act = new Action(engine.icon, tr("Search \"%1 ..\" with %2").arg(selectedText, engine.name));
     connect(act, SIGNAL(triggered()), this, SLOT(searchSelectedText()));
     connect(act, SIGNAL(ctrlTriggered()), this, SLOT(searchSelectedTextInBackgroundTab()));
@@ -1261,14 +1266,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
 void WebView::loadRequest(const LoadRequest &req)
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
     QWebEngineView::load(req.webRequest());
-#else
-    if (req.operation() == LoadRequest::GetOperation)
-        load(req.url());
-    else
-        page()->runJavaScript(Scripts::sendPostData(req.url(), req.data()), WebPage::SafeJsWorld);
-#endif
 }
 
 bool WebView::eventFilter(QObject *obj, QEvent *event)
