@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - Qt web browser
-* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -46,10 +46,6 @@ WebTab::SavedTab::SavedTab()
 
 WebTab::SavedTab::SavedTab(WebTab* webTab)
 {
-    if (webTab->url().toString() == QL1S("qupzilla:restore")) {
-        return;
-    }
-
     title = webTab->title();
     url = webTab->url();
     icon = webTab->icon(true);
@@ -60,7 +56,7 @@ WebTab::SavedTab::SavedTab(WebTab* webTab)
 
 bool WebTab::SavedTab::isValid() const
 {
-    return !url.isEmpty();
+    return !url.isEmpty() || !history.isEmpty();
 }
 
 void WebTab::SavedTab::clear()
@@ -231,10 +227,10 @@ QUrl WebTab::url() const
     }
 }
 
-QString WebTab::title() const
+QString WebTab::title(bool allowEmpty) const
 {
     if (isRestored()) {
-        return m_webView->title();
+        return m_webView->title(allowEmpty);
     }
     else {
         return m_savedTab.title;
@@ -271,12 +267,15 @@ void WebTab::setZoomLevel(int level)
 
 void WebTab::detach()
 {
+    Q_ASSERT(m_window);
     Q_ASSERT(m_tabBar);
 
     // Remove icon from tab
-    m_tabBar->setTabButton(tabIndex(), m_tabBar->iconButtonPosition(), 0);
+    m_tabBar->setTabButton(tabIndex(), m_tabBar->iconButtonPosition(), nullptr);
+    m_tabIcon->setParent(this);
 
     // Remove the tab from tabbar
+    m_window->tabWidget()->removeTab(tabIndex());
     setParent(0);
     // Remove the locationbar from window
     m_locationBar->setParent(this);
@@ -381,19 +380,6 @@ void WebTab::restoreTab(const WebTab::SavedTab &tab)
         m_tabBar->setTabText(index, tab.title);
         m_locationBar->showUrl(tab.url);
         m_tabIcon->updateIcon();
-
-        if (!tab.url.isEmpty()) {
-            QColor col = m_tabBar->palette().text().color();
-            QColor newCol = col.lighter(250);
-
-            // It won't work for black color because (V) = 0
-            // It won't also work for white, as white won't get any lighter
-            if (col == Qt::black || col == Qt::white) {
-                newCol = Qt::gray;
-            }
-
-            m_tabBar->overrideTabTextColor(index, newCol);
-        }
     }
     else {
         // This is called only on restore session and restoring tabs immediately
@@ -440,7 +426,7 @@ void WebTab::showNotification(QWidget* notif)
 
 void WebTab::loadStarted()
 {
-    if (m_tabBar && m_webView->isTitleEmpty()) {
+    if (m_tabBar && m_webView->title(/*allowEmpty*/true).isEmpty()) {
         m_tabBar->setTabText(tabIndex(), tr("Loading..."));
     }
 }
@@ -467,24 +453,21 @@ void WebTab::slotRestore()
 {
     Q_ASSERT(m_tabBar);
 
+    if (isRestored()) {
+        return;
+    }
+
     p_restoreTab(m_savedTab);
     m_savedTab.clear();
-
-    m_tabBar->restoreTabTextColor(tabIndex());
 }
 
 void WebTab::tabActivated()
 {
-
-    if (!isRestored()) {
-        // When session is being restored, restore the tab immediately
-        if (mApp->isRestoring()) {
-            slotRestore();
-        }
-        else {
-            QTimer::singleShot(0, this, SLOT(slotRestore()));
-        }
+    if (isRestored()) {
+        return;
     }
+
+    QTimer::singleShot(0, this, SLOT(slotRestore()));
 }
 
 void WebTab::resizeEvent(QResizeEvent *event)
