@@ -1,6 +1,6 @@
 /* ============================================================
 * StatusBarIcons - Extra icons in statusbar for QupZilla
-* Copyright (C) 2013-2017 David Rosca <nowrep@gmail.com>
+* Copyright (C) 2013-2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@ void SBI_JavaScriptIcon::toggleJavaScript()
     }
 
     bool current = testCurrentPageWebAttribute(QWebEngineSettings::JavascriptEnabled);
-    currentPage()->setJavaScriptEnabled(!current);
+    setCurrentPageWebAttribute(QWebEngineSettings::JavascriptEnabled, !current);
 
     m_window->weView()->reload();
 
