@@ -1,6 +1,7 @@
 /* ============================================================
 * TabManager plugin for QupZilla
-* Copyright (C) 2013-2017  S. Razi Alavizadeh <s.r.alavizadeh@gmail.com>
+* Copyright (C) 2013-2017 S. Razi Alavizadeh <s.r.alavizadeh@gmail.com>
+* Copyright (C)      2018 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -471,9 +472,6 @@ void TabManagerWidget::processActions()
             }
 
             if (command == "closeSelection") {
-                if (webTab->url().toString() == "qupzilla:restore") {
-                    continue;
-                }
                 selectedTabs.insertMulti(mainWindow, webTab);
             }
             else if (command == "detachSelection" || command == "bookmarkSelection") {
@@ -540,12 +538,12 @@ static void detachTabsTo(BrowserWindow* targetWindow, const QHash<BrowserWindow*
         foreach (WebTab* webTab, tabs) {
             mainWindow->tabWidget()->detachTab(webTab);
 
-            if (mainWindow && mainWindow->tabWidget()->count() == 0) {
+            if (mainWindow && mainWindow->tabCount() == 0) {
                 mainWindow->close();
                 mainWindow = 0;
             }
 
-            targetWindow->tabWidget()->addView(webTab);
+            targetWindow->tabWidget()->addView(webTab, Qz::NT_NotSelectedTab);
         }
     }
 }
@@ -554,7 +552,7 @@ void TabManagerWidget::detachSelectedTabs(const QHash<BrowserWindow*, WebTab*> &
 {
     if (tabsHash.isEmpty() ||
             (tabsHash.uniqueKeys().size() == 1 &&
-             tabsHash.size() == tabsHash.keys().at(0)->tabWidget()->count())) {
+             tabsHash.size() == tabsHash.keys().at(0)->tabCount())) {
         return;
     }
 
